/*
 * commande de 4 aiguillages sur Arduino UNO/NANO sans DCC avec bouton et led pour TCO
 * Les aiguillages sont commandes par des sorties sur les broches a partir de la broche 3 par paire (aiguillage 1 =  (3,4) etc...)
 * ces sorties peuvent etre utilisees directement pour commander des ULN2803 ou les inputs de relais ou L298N 
 * les bouton sont traites avec anti-rebond
 * Les led indiquent la position de l'aiguillage sur le TCO
 * Configuration :
 * aiguillage 1 : bobine1 : broche 3, bobine2 : broche 4
 * aiguillage 2 : bobine1 : broche 5, bobine2 : broche 6
 * aiguillage 3 : bobine1 : broche 7, bobine2 : broche 8
 * aiguillage 4 : bobine1 : broche 9, bobine2 : broche 10
 * led aiguillage 1 : broche 11
 * led aiguillage 2 : broche 12
 * led aiguillage 3 : broche 13
 * led aiguillage 4 : broche A0
 * bouton aiguillage 1 : broche A1
 * bouton aiguillage 2 : broche A2
 * bouton aiguillage 3 : broche A3
 * bouton aiguillage 4 : broche A4
 * 
 */

/******************************************************************************************
 * 
 * LES PARAMETRES DE CONFIGURATION a definir par l'utilisateur
 * CONSOLE : pour avoir le deroulement du programme, si pas necessaire supprimer la ligne
 * FIRST_ID_DCC : premiere adresse DCC de depart, premier aiguillage (FIRST_ID_DCC sur broche 3), second aiguillage (FIRST_ID_DCC+1 sur broche 4, etc..)
 * NB_TURNOUT : le nombre d'aiguillages
 * 
 *********************************************************************************************/
 
 #define CONSOLE                         // sortie console du deroulement
 #define DEBIT_CONSOLE    115200         // le debit de la console
 #define ACTIVATION        250           // duree d'activation des bobines en ms ( amodifier si necessaire) 

 /***********************************************************************************************
  * 
  * LES PARAMETRES INTERNES
  * 
  * ************************************************************************************************/

#define TRAVAIL   LOW         // LOW pour Relais, L298N, L9110; HIGH pour ULN2803A
#define REPOS     HIGH        // HIGH pour Relais, L298N, L9110; LOW pour ULN2803A
#define BOBINE1   HIGH        // position sur bobine1
#define BOBINE2   LOW         // position sur bobune2

#define DELAI_ANTI_REBOND  50

/////////////////// LES AIGUILLAGES  ///////////////////////////////////////////////////////

#define FIRST_PIN         3             // broche de depart des aiguillages

//// STRUCTURE D'UN AIGUILLAGE ////

struct TurnOut {
  int position_actuelle;                    // etat, bobine1 ou bobine2
  int bobine1;                              // broche bobine 1
  int bobine2;                              // broche bobine 2
  int bouton;                               // broche du bouton TCO a bascule
  int etat_bouton;
  int ledBobine;                            // broche de la Led TCO pour les bobines
  boolean demande_activation;     // demande d'activation, bouton bascule
};
#define NB_TURNOUT        4       // le nombre  d'aiguillages
TurnOut turnout[NB_TURNOUT];      // l'ensemble des aiguillages

/********************************************************************
 * 
 * METHODE APPELEE SI UNE DEMANDE DE MISE A JOUR PAR LE BOUTON
 * Determine periodiquemùent les bobines a activer
 * 
 *******************************************************************/
 
void activation_aiguillage(int i) {
    int etat = turnout[i].position_actuelle;
    if (turnout[i].demande_activation == true)         // si l'aiguillage i est en attente d'activation
    {
      if (turnout[i].position_actuelle==BOBINE1){                   // bobine 1  --> bobine 2
                      digitalWrite(turnout[i].bobine2,TRAVAIL);     // activation
                      delay(ACTIVATION);                         // duree de l'activation
                      digitalWrite(turnout[i].bobine2,REPOS);      // arret de l'activation
                      digitalWrite(turnout[i].ledBobine, LOW);    // led bobine2 allumee
                      etat = BOBINE2;
                      #ifdef CONSOLE
                        Serial.print("activation -> aiguillage");Serial.print(i);Serial.println(" : bobine2");
                      #endif
                      }
        else{                                                      // bobine 2 --> bobine 1
                      digitalWrite(turnout[i].bobine1,TRAVAIL);      // activation
                      delay(ACTIVATION);                          // duree de l'activation
                      digitalWrite(turnout[i].bobine1,REPOS);       // arret de l'activation
                      digitalWrite(turnout[i].ledBobine, HIGH);     // led bobine1 allumee
                      etat = BOBINE1;
                      #ifdef CONSOLE
                        Serial.print("activation -> aiguillage");Serial.print(i);Serial.println(" : bobine1");
                      #endif
              }
    }
   turnout[i].position_actuelle = etat;                 // etat de la bascule
  turnout[i].demande_activation = false;                // l'aiguillage a etait traite
 }


/************************************************************************************* 
 *  
 *  METHODE de test des boutons avec anti rebond 
 *  determine si un aiguillage est concerne par l'appui d'un bouton
 *  
 ***********************************************************************************/

boolean boutonTest(int i)
{   turnout[i].demande_activation = false;
    int etat = digitalRead(turnout[i].bouton);
    int tempo = millis();
    while ((millis() - tempo) < DELAI_ANTI_REBOND) {}
    if (etat == LOW){turnout[i].demande_activation = true;}
  return turnout[i].demande_activation;
}

/**********************************************************************************************
 * 
 * le setup
 * initialise les aiguillages
 * 
 ******************************************************************************************/

void setup() {
  int decal = 0;
  for (int i=0; i<NB_TURNOUT; i++){             // pour tous les aiguillages
    turnout[i].demande_activation = false;          // pas de demande d'activation
    turnout[i].position_actuelle = BOBINE1;            // par defaut sur bobine 1
    
    turnout[i].bobine1 = decal+FIRST_PIN;           // numero de broche de la bobine 1
    pinMode(turnout[i].bobine1, OUTPUT);            // bobine 1   en sortie 
    digitalWrite(turnout[i].bobine1, REPOS);          // bobine 1 au repos
   
    
    turnout[i].bobine2 = 1+decal+FIRST_PIN;         // numero de la broche de la bobine 2
    pinMode(turnout[i].bobine2, OUTPUT);            // bobine 2  en sortie  
    digitalWrite(turnout[i].bobine2, REPOS);          // bobine 2 au repos
    
    
    turnout[i].ledBobine = FIRST_PIN + i +8;           // les led du TCO
    pinMode(turnout[i].ledBobine, OUTPUT);
    digitalWrite(turnout[i].ledBobine,HIGH);        // bobine 1 allumee par defaut
    delay(ACTIVATION);
     digitalWrite(turnout[i].ledBobine,LOW);
     delay(ACTIVATION);
     digitalWrite(turnout[i].ledBobine,HIGH);
          
    turnout[i].bouton = FIRST_PIN + i + 12;          // le bouton du TCO
    pinMode(turnout[i].bouton,INPUT_PULLUP);
    turnout[i].etat_bouton = HIGH;
   
    decal+=2;                                   // on passe aux aiguillages suivants  
  }
  
  #ifdef CONSOLE
   Serial.begin(DEBIT_CONSOLE);
   for (int i=0; i<NB_TURNOUT; i++){
    Serial.print("aiguillage");Serial.println(i); 
    Serial.print("\t bobine 1 sur broche : ");Serial.println(turnout[i].bobine1);
    Serial.print("\t bobine 2 sur broche : ");Serial.println(turnout[i].bobine2);
    Serial.print("\t bouton TCO sur broche : ");Serial.println(turnout[i].bouton);
    Serial.print("\t Led TCO sur broche : ");Serial.println(turnout[i].ledBobine);
  }
  #endif
}

/*************************************************************************
 * boucle
 * 
 ******************************************************************/
 
void loop() {
  for (int i = 0;i<NB_TURNOUT;i++) {if (boutonTest(i))  activation_aiguillage(i);}
}
  
